#include <windows.h>
#include <stdio.h>
#include "c:\98ddk\inc\win98\setupapi.h"	// VC++ 5 one is out of date
#include "initguid.h"
#include "..\guid.h"

HANDLE GetDeviceViaInterface( GUID* pGuid, DWORD instance);

int main(int argc, char* argv[])
{
	// Open device
	HANDLE hHiImWdm = GetDeviceViaInterface((LPGUID)&WDM_GUID,0);
	if( hHiImWdm==NULL)
	{
		printf("Could not find open HiImWdm device\n");
		return 1;
	}
	printf("Opened OK\n");

	// Read what's left in buffer
	DWORD TxdBytes;
	ULONG Rvalue = 0;
	if( !ReadFile( hHiImWdm, &Rvalue, 4, &TxdBytes, NULL))
		printf("Could not read value\n");
	else if( TxdBytes==4)
		printf("Read successfully read stored value of 0x%X\n",Rvalue);
	else
		printf("Wrong number of bytes read: %d\n",TxdBytes);

	// Write
	ULONG Wvalue = 0xabcdef01;
	if( !WriteFile( hHiImWdm, &Wvalue, 4, &TxdBytes, NULL))
		printf("Could not write %X\n",Wvalue);
	else if( TxdBytes==4)
		printf("Write succeeded\n");
	else
		printf("Wrong number of bytes written: %d\n",TxdBytes);

	// Read
	Rvalue = 0;
	if( !ReadFile( hHiImWdm, &Rvalue, 4, &TxdBytes, NULL))
		printf("Could not read value\n");
	else if( TxdBytes==4)
	{
		if( Rvalue==Wvalue)
			printf("Read succeeded\n");
		else
			printf("Read succeeded, but got wrong value: %X",Rvalue);
	}
	else
		printf("Wrong number of bytes read: %d\n",TxdBytes);

	// Check duff write fails
	if( !WriteFile( hHiImWdm, &Wvalue, 5, &TxdBytes, NULL))
		printf("Duff Write correctly failed with error %d\n",GetLastError());
	else
		printf("Duff Write unexpectedly succeeded\n");

	// Close device
	CloseHandle(hHiImWdm);
	return 0;
}

HANDLE GetDeviceViaInterface( GUID* pGuid, DWORD instance)
{
	// Get handle to relevant device information set
	HDEVINFO info = SetupDiGetClassDevs(pGuid, NULL, NULL, DIGCF_PRESENT | DIGCF_INTERFACEDEVICE);
	if(info==INVALID_HANDLE_VALUE)
	{
		printf("No HDEVINFO available for this GUID\n");
		return NULL;
	}

	// Get interface data for the requested instance
	SP_INTERFACE_DEVICE_DATA ifdata;
	ifdata.cbSize = sizeof(ifdata);
	if(!SetupDiEnumDeviceInterfaces(info, NULL, pGuid, instance, &ifdata))
	{
		printf("No SP_INTERFACE_DEVICE_DATA available for this GUID instance\n");
		SetupDiDestroyDeviceInfoList(info);
		return NULL;
	}

	// Get size of symbolic link name
	DWORD ReqLen;
	SetupDiGetDeviceInterfaceDetail(info, &ifdata, NULL, 0, &ReqLen, NULL);
	PSP_INTERFACE_DEVICE_DETAIL_DATA ifDetail = (PSP_INTERFACE_DEVICE_DETAIL_DATA)(new char[ReqLen]);
	if( ifDetail==NULL)
	{
		SetupDiDestroyDeviceInfoList(info);
		return NULL;
	}

	// Get symbolic link name
	ifDetail->cbSize = sizeof(SP_INTERFACE_DEVICE_DETAIL_DATA);
	if( !SetupDiGetDeviceInterfaceDetail(info, &ifdata, ifDetail, ReqLen, NULL, NULL))
	{
		SetupDiDestroyDeviceInfoList(info);
		delete ifDetail;
		return NULL;
	}

	printf("Symbolic link is %s\n",ifDetail->DevicePath);
	// Open file
	HANDLE rv = CreateFile( ifDetail->DevicePath, 
		GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	delete ifDetail;
	SetupDiDestroyDeviceInfoList(info);
	return rv;
}
