/////////////////////////////////////////////////////////////////////////////
//	Copyright (C) 1998 Chris Cant, PHD Computer Consultants Ltd
/////////////////////////////////////////////////////////////////////////////

#include "hiimwdm.h"

#pragma code_seg("PAGE")	// start PAGE section

//////////////////////////////////////////////////////////////////////////////

extern UCHAR SharedMemory[];

#if DBG
WMIGUIDREGINFO HiImWdmWmiGuidList[1] =
{
    { &WMI_GUID, 1, 0 },
};
#endif // DBG

//////////////////////////////////////////////////////////////////////////////

void RegisterWmi( IN PDEVICE_OBJECT fdo)
{
#if DBG
	PWDM_DEVICE_EXTENSION dx=(PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;

	dx->WmiLibInfo.GuidCount = 1;
	dx->WmiLibInfo.GuidList = HiImWdmWmiGuidList;

	dx->WmiLibInfo.QueryWmiRegInfo = QueryWmiRegInfo;
	dx->WmiLibInfo.QueryWmiDataBlock = QueryWmiDataBlock;
	dx->WmiLibInfo.SetWmiDataBlock = SetWmiDataBlock;
	dx->WmiLibInfo.SetWmiDataItem = SetWmiDataItem;
	dx->WmiLibInfo.ExecuteWmiMethod = NULL;
	dx->WmiLibInfo.WmiFunctionControl = NULL;

	IoWMIRegistrationControl(fdo, WMIREG_ACTION_REGISTER);
#endif // DBG
}

//////////////////////////////////////////////////////////////////////////////

void DeregisterWmi( IN PDEVICE_OBJECT fdo)
{
#if DBG
	IoWMIRegistrationControl(fdo, WMIREG_ACTION_DEREGISTER);
#endif // DBG
}

//////////////////////////////////////////////////////////////////////////////

NTSTATUS WdmSystemControl(
	IN PDEVICE_OBJECT fdo,
	IN PIRP Irp)
{
	PWDM_DEVICE_EXTENSION dx = (PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;
	NTSTATUS status;
#if DBG
	SYSCTL_IRP_DISPOSITION disposition;

	status = WmiSystemControl(&dx->WmiLibInfo, fdo, Irp, &disposition);
    switch(disposition)
    {
    case IrpProcessed:
        // This irp has been processed and may be completed or pending.
        break;
    
    case IrpNotCompleted:
        // This irp has not been completed, but has been fully processed.
        // we will complete it now
        IoCompleteRequest(Irp, IO_NO_INCREMENT);                
        break;
    
    case IrpForward:
    case IrpNotWmi:
        // This irp is either not a WMI irp or is a WMI irp targetted
        // at a device lower in the stack.
#endif // DBG
        IoSkipCurrentIrpStackLocation(Irp);
        status = IoCallDriver(dx->NextDevice, Irp);
#if DBG
        break;
                                
    default:
        ASSERT(FALSE);
    }
#endif // DBG
    
    return status;
}

//////////////////////////////////////////////////////////////////////////////

#if DBG

NTSTATUS FailWMIRequest(
	IN PDEVICE_OBJECT fdo,
	IN PIRP Irp,
	IN ULONG GuidIndex)
{
	PWDM_DEVICE_EXTENSION dx = (PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;
	NTSTATUS status;

	if(GuidIndex==0)
		status = STATUS_INVALID_DEVICE_REQUEST;
	else
		status = STATUS_WMI_GUID_NOT_FOUND;

	status = WmiCompleteRequest(fdo, Irp, status, 0, IO_NO_INCREMENT);

	return status;
}

//////////////////////////////////////////////////////////////////////////////

NTSTATUS SetWmiDataItem(
	IN PDEVICE_OBJECT fdo, IN PIRP Irp,
	IN ULONG GuidIndex, IN ULONG InstanceIndex,
	IN ULONG DataItemId,
	IN ULONG BufferSize, IN PUCHAR PBuffer)
{
   return FailWMIRequest(fdo, Irp, GuidIndex);
}

//////////////////////////////////////////////////////////////////////////////

NTSTATUS SetWmiDataBlock(
	IN PDEVICE_OBJECT fdo, IN PIRP Irp,
	IN ULONG GuidIndex, IN ULONG InstanceIndex,
	IN ULONG BufferSize,
	IN PUCHAR PBuffer)
{
   return FailWMIRequest(fdo, Irp, GuidIndex);
}

//////////////////////////////////////////////////////////////////////////////

NTSTATUS QueryWmiDataBlock(
	IN PDEVICE_OBJECT fdo, IN PIRP Irp,
	IN ULONG GuidIndex, 
	IN ULONG InstanceIndex,
	IN ULONG InstanceCount,
	IN OUT PULONG InstanceLengthArray,
	IN ULONG OutBufferSize,
	OUT PUCHAR PBuffer)
{
	PWDM_DEVICE_EXTENSION dx = (PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;
	NTSTATUS status;
	ULONG size = 0;

	switch (GuidIndex)
	{
	case 0:
	{
		ULONG SymLinkNameLen = dx->ifSymLinkName.Length;
		size = sizeof(ULONG)+sizeof(USHORT)+SymLinkNameLen;

		// Check output buffer size
		if (OutBufferSize<size)
		{
			status = STATUS_BUFFER_TOO_SMALL;
			break;
		}

		// Store uint32 BufferFirstWord
		*(ULONG *)PBuffer = *(ULONG*)SharedMemory;
		(UCHAR *)PBuffer += sizeof(ULONG);

		// Store string SymbolicLinkName as counted Unicode
		*(USHORT *)PBuffer = (USHORT)SymLinkNameLen;
		(UCHAR *)PBuffer += sizeof(USHORT);
		RtlCopyMemory(PBuffer, dx->ifSymLinkName.Buffer, SymLinkNameLen);

		// Store total size
		*InstanceLengthArray = size;

		status = STATUS_SUCCESS;

		break;
	}
	default:
		status = STATUS_WMI_GUID_NOT_FOUND;
		break;
	}

	status = WmiCompleteRequest( fdo, Irp, status, size, IO_NO_INCREMENT);

	return status;
}

//////////////////////////////////////////////////////////////////////////////

#define MofResourceNameText L"MofResource"

NTSTATUS QueryWmiRegInfo(
	IN PDEVICE_OBJECT fdo, OUT PULONG PRegFlags,
	OUT PUNICODE_STRING PInstanceName,
	OUT PUNICODE_STRING *PRegistryPath,
	OUT PUNICODE_STRING MofResourceName,
	OUT PDEVICE_OBJECT *Pdo)
{
	PWDM_DEVICE_EXTENSION dx = (PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;

	*PRegFlags = WMIREG_FLAG_INSTANCE_PDO;
	*PRegistryPath = &HiImWdmRegistryPath;
	RtlInitUnicodeString(MofResourceName, MofResourceNameText);
	*Pdo = dx->pdo;

   return STATUS_SUCCESS;
}
#endif // DBG

#pragma code_seg()	// end PAGE section
